use strict;
use warnings;
use Test::More;

# generated by Dist::Zilla::Plugin::Test::PodSpelling 2.007005
use Test::Spelling 0.12;
use Pod::Wordlist;


add_stopwords(<DATA>);
all_pod_files_spelling_ok( qw( bin lib ) );
__DATA__
CheckFor
Contributors
CoreCounter
CorrectBranch
DROLSKY
DROLSKY's
Dave
Dist
Fowler
Git
License
MakeMaker
Mark
PayPal
Plugin
PluginBundle
Pod
Role
Rolsky
Rolsky's
RunExtraTests
TidyAll
Weaver
WeaverConfig
Zilla
autarch
drolsky
ini
lib
mailmap
mark
optimizations
perlcriticrc
perltidyrc
tidyall
